/******************************************************************************
*
* project name:    FIRE
* initial date:    16/06/2000
* author:          thomas.nussbaumer@peem.com
* description:     fire effect demonstration
*
* !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
* NOTE: depends on TIGCC Standard Library 2.x to compile
* !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
*
* compile with: tigcc -O2 fire.c
*
* -----------------------------------------------------------------------------
* Revision History
* -----------------------------------------------------------------------------
*
* v1.01: 28/06/2000
*        height of backbuffer modified (previously misstyped) and documentation
*        of InitBackBuffer extended to clearify the used mechanism
*
* v1.00: 16/06/2000
*        initial version
*
*******************************************************************************/

#define SAVE_SCREEN      // restore screen after program end
#define USE_GRAY_PATCH   // use grayscale patch

#include <nostub.h>
#include <all.h>

/*---------------------------------------------------------------------------*/
/* global variables                                                          */
/*---------------------------------------------------------------------------*/
int            _ti92plus;          // needed to produce ti92plus code
int            _ti89;              // needed to produce ti89 code

unsigned char* plane1        = 0;  // startadress of grayscale bitplane 1
unsigned char* plane2        = 0;  // startadress of grayscale bitplane 2
unsigned char* backbuffer[4] = {}; // used as offscreen buffer to generate image


/*===========================================================================*/
/* allocates memory for backup buffer and clears contents                    */
/*                                                                           */
/* NOTE: all 4 backbuffers (2*2planes) are allocated in one piece and        */
/*       splitted up afterwards to save system handles.                      */
/*       if you allocate each plane itself you would to free each plane      */
/*       separately,too. Due to the fact that we just allocate one large     */
/*       piece of memory we have to free just this one at the end of the     */
/*       program                                                             */
/*===========================================================================*/
int InitBackBuffer() {
    int i;
    
    if ((backbuffer[0] = (unsigned char*)malloc(30*128*4))==0) return 0;

    for (i=1;i<4;i++) backbuffer[i] = backbuffer[0]+i*30*128;
    memset(backbuffer[0],0,128*30*4);
    return 1;
}


/*===========================================================================*/
/* copies offscreen image starting from line 40 to visible bitplanes         */
/* (just from line 40 so that the title won't be overwritten!)               */ 
/*===========================================================================*/
void BlitScreen() {
    int y;
        
    for (y=40;y<LCD_HEIGHT;y++) {
        memcpy(plane1+y*30,backbuffer[2] + y*30,LCD_LINE_BYTES);
        memcpy(plane2+y*30,backbuffer[3] + y*30,LCD_LINE_BYTES);
    }

    memcpy(backbuffer[0],backbuffer[2],128*30);
    memcpy(backbuffer[1],backbuffer[3],128*30);
}


/*===========================================================================*/
/* fillup bottom lines in first backbuffer                                   */
/*===========================================================================*/
void FillBottomLines() {
    int          height = LCD_HEIGHT;
    int          x;

    memset(backbuffer[0] + (height-3)*30,0xff,90);
    memset(backbuffer[1] + (height-3)*30,0xff,90);

    for (x=0;x<30;x++) {
        *(backbuffer[0] + (height-4)*30+x) = (unsigned char)0xff;
        *(backbuffer[1] + (height-4)*30+x) = (unsigned char)random(256);
    }
}


/*===========================================================================*/
/* copies first backbuffer into second backbuffer with one line offset so    */
/* it looks like as scrolling upwards and modify some pixels randomly        */
/* afterwards                                                                */
/*===========================================================================*/
void ScrollUp() {
    int y;
    
    unsigned char *p1;
    unsigned char *p2;
    
    int value;
    
    memcpy(backbuffer[2],backbuffer[0]+30,127*30);
    memcpy(backbuffer[3],backbuffer[1]+30,127*30);
    
    for (p1=backbuffer[2]+LCD_HEIGHT*2*30/3,
         p2=backbuffer[3]+LCD_HEIGHT*2*30/3;
         p1<backbuffer[2]+(LCD_HEIGHT-3)*30;
         p1++,p2++) 
    {
        value = random(700);
        if (value < 255) *p1 &= (unsigned char)value;
        value = random(700);
        if (value < 255) *p2 &= (unsigned char)value;
    }

}


/*===========================================================================*/
/* main routine: where all the fun starts ...                                */
/*===========================================================================*/
void _main(void) {
    char* s1 = "FIRE EFFECT DEMO";
    char* s2 = "(c) TI-Chess Team 06/2000";
    int   i;
    
    //---------------------------------------------
    // try to allocate memory for offscreen buffers
    //---------------------------------------------
    if (!InitBackBuffer()) {
        ClearScreen();
        DrawStr(0,0,"ERROR: too less mem",A_REPLACE);
        ngetchx();
        return;
    }
    
    //---------------------------------------------
    // turn on grayscale graphics subsystem
    //---------------------------------------------
    if (!GrayMode(GRAY_ON)) {
        free(backbuffer[0]);
        return;
    }

    //---------------------------------------------
    // clear both planes 
    //---------------------------------------------
    SetPlane(0);
    ClearScreen();
    SetPlane(1);
    ClearScreen();

    //---------------------------------------------
    // fetch start addresses of planes
    //---------------------------------------------
    plane1 = GetPlane(0);
    plane2 = GetPlane(1);

    //---------------------------------------------
    // write title
    //---------------------------------------------
    for (i=0;i<2;i++) {
        SetPlane(i);
        FontSetSys(F_8x10);
        DrawStr((ScrRect->xy.x1-DrawStrWidth(s1,F_8x10))/2,10,s1,A_NORMAL);
        FontSetSys(F_4x6);
        DrawStr((ScrRect->xy.x1-DrawStrWidth(s2,F_4x6))/2,30,s2,A_NORMAL);
    }

    //---------------------------------------------
    // main loop
    //---------------------------------------------
    do {
        FillBottomLines();        
        ScrollUp();
        BlitScreen();
    } 
    while (!kbhit());
    ngetchx();

    //---------------------------------------------
    // cleanup and bye =8-)
    //---------------------------------------------
    GrayMode(GRAY_OFF);
    free(backbuffer[0]);
}

