/******************************************************************************
*
* project name: Shuffle Effect Demo
* file name:    shuffle.c 
* initial date: 02/11/2000
* author:       thomas.nussbaumer@gmx.net
* description:  splits homescreen into tiles and shuffles them
*
* compile with: tigcc -O2 shuffle.c
*
* $Id: shuffle.c,v 1.1 2000/11/04 19:29:00 Thomas Nussbaumer Exp $
*
******************************************************************************/

#include <tigcclib.h>

int _ti92plus;
int _ti89;

#define MAX_TILESIZE 5

int nrcols  = 0;
int nrlines = 0;
int sizex   = 0;
int sizey   = 0;

typedef struct {
    unsigned int NumRows, NumCols;
    unsigned char Data[MAX_TILESIZE*MAX_TILESIZE];  // maybe this is too large, but I don't care
} MYBITMAP;


//=============================================================================
// swap two tiles using source as screenbuffer
//=============================================================================
void SwapFields(unsigned char* source,int field1,int field2) {
    MYBITMAP bm1;
    MYBITMAP bm2;
    SCR_RECT rect1;
    SCR_RECT rect2;
    SCR_RECT fullscreen;
    
    //--------------------------------------------------------------------
    // this is the shortest and fasted way to fill the fullscreen variable
    //--------------------------------------------------------------------
    fullscreen.l = (((unsigned long)239)<<8) | 127;
    
    //------------------------------
    // set size of BITMAP structures
    //------------------------------
    bm1.NumRows = bm2.NumRows = sizex;
    bm1.NumCols = bm2.NumCols = sizey;

    //----------------------------------------------------
    // fill up the rectangles and get the fill the bitmaps
    //----------------------------------------------------
    rect1.xy.x0=(field1 % nrcols)*sizex;
    rect1.xy.y0=(field1 / nrcols)*sizey;
    rect1.xy.x1=rect1.xy.x0+sizex-1;
    rect1.xy.y1=rect1.xy.y0+sizey-1;

    rect2.xy.x0=(field2 % nrcols)*sizex;
    rect2.xy.y0=(field2 / nrcols)*sizey;
    rect2.xy.x1=rect2.xy.x0+sizex-1;
    rect2.xy.y1=rect2.xy.y0+sizey-1;

    BitmapGet(&rect1,(BITMAP*)&bm1);
    BitmapGet(&rect2,(BITMAP*)&bm2);

    //--------------------------------------------------------------
    // put the bitmaps in switched order into the given screenbuffer
    //--------------------------------------------------------------
    BitmapPut(rect1.xy.x0,rect1.xy.y0, (BITMAP*)&bm2, &fullscreen, A_REPLACE); 
    BitmapPut(rect2.xy.x0,rect2.xy.y0, (BITMAP*)&bm1, &fullscreen, A_REPLACE); 
}


//=============================================================================
// executes the screen tile shuffle
//=============================================================================
void Shuffle() {
    int i,j,k;
    LCD_BUFFER buf;

    //------------------------------
    // get original screen content
    //------------------------------
    LCD_save(buf);
        
    randomize();
    
    //-----------------------------------------------------------------------
    // tell the AMS that we are using our own screen buffer for manipulations
    //-----------------------------------------------------------------------
    PortSet(buf,239,127); 

    //-------------------------------------
    // swap tiles until user gets bored ...
    //-------------------------------------
    while (!kbhit()) {
        i = random(nrlines); // get a random line number
        j = random(nrcols);  // get a random column number
        k = random(4);       // get a random direction to switch
        
        if (k==0) {
            //------------------------------
            // switch with tile above
            //------------------------------
            if (!i) i++;
            SwapFields(buf,i*nrcols+j,(i-1)*nrcols+j);
        }
        else if (k==1) {
            //------------------------------
            // switch with tile below
            //------------------------------
            if (i==nrlines-1) i--;
            SwapFields(buf,i*nrcols+j,(i+1)*nrcols+j);
        }
        else if (k==2) {
            //------------------------------
            // switch with tile to the left
            //------------------------------
            if (!j) j++;
            SwapFields(buf,i*nrcols+j,i*nrcols+j-1);
        }
        else if (k==3) {
            //------------------------------
            // switch with tile to the right
            //------------------------------
            if (j==nrcols-1) j--;
            SwapFields(buf,i*nrcols+j,i*nrcols+j+1);
        }
        //-----------------------------
        // draw modified screen content 
        //-----------------------------
        LCD_restore(buf);
    }
    //----------------------------------------------------
    // clear keyboard buffer
    //----------------------------------------------------
    GKeyFlush();
    
    PortRestore();
}



//=============================================================================
// where all the fun starts as usual ...
//=============================================================================
void _main(void) {
    LCD_BUFFER   savescreen;   // NOTE: I do not use the SAVESCREEN_PATCH, 
                               //       because I want to output a message to
                               //       the statusline at the end

    //-----------------------------------------
    // set size of tiles depending on calctype
    // DONT FORGET ADAPTING MAX_TILESIZE if you
    // change these settings !!!!
    //-----------------------------------------
    if (TI89) {
        sizex   = 5;  
        sizey   = 5;
    }
    else {
        sizex   = 5;
        sizey   = 4;
    }

    nrcols  = LCD_WIDTH/sizex;
    nrlines = LCD_HEIGHT/sizey;
    
    LCD_save(savescreen);
    Shuffle();
    LCD_restore(savescreen);
    
    ST_showHelp("TICT Production 2000");
}

//#############################################################################
// Revision History
//#############################################################################
//
// $Log: shuffle.c,v $
// Revision 1.1  2000/11/04 19:29:00  Thomas Nussbaumer
// initial version
//
//
